from pathlib import Path

import pytest

from wunderkafka.serdes.json import HAS_JSON_SCHEMA
from wunderkafka.serdes.schemaless.string.deserializers import StringDeserializer

if not HAS_JSON_SCHEMA:
    pytest.skip("skipping json-schema-only tests", allow_module_level=True)
from wunderkafka.tests import TestConsumer, TestHTTPClient
from wunderkafka.serdes.headers import ConfluentClouderaHeadersHandler
from wunderkafka.tests.consumer import Message
from wunderkafka.schema_registry import SimpleCache, ConfluentSRClient
from wunderkafka.consumers.constructor import HighLevelDeserializingConsumer
from tests.integration.confluent.conftest import Msg
from wunderkafka.serdes.json.deserializers import JSONDeserializer

MESSAGE = Msg(
    payload=b'{"id": "714fc713-37ff-4477-9157-cb4f14b63e1a", "path": "/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3"}',
    # noqa: E501
    deserialized={
        "id": "714fc713-37ff-4477-9157-cb4f14b63e1a",
        "path": "/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3",
    },
)

HEADERS = (b"\x00\x00\x00\x07<",)


@pytest.mark.parametrize("header", list(HEADERS))
def test_consume_moving_parts(sr_root_existing: Path, topic: str, header: bytes) -> None:
    msg = Message(topic, value=MESSAGE.serialized(header), key=b"714fc713-37ff-4477-9157-cb4f14b63e1a")
    consumer = HighLevelDeserializingConsumer(
        consumer=TestConsumer([msg]),
        schema_registry=ConfluentSRClient(TestHTTPClient(sr_root_existing), SimpleCache()),
        headers_handler=ConfluentClouderaHeadersHandler().parse,
        value_deserializer=JSONDeserializer(),
        key_deserializer=StringDeserializer(),
    )

    consumer.subscribe([topic], from_beginning=True)

    messages: list[Message] = consumer.consume()
    [message] = messages
    assert message.key() == "714fc713-37ff-4477-9157-cb4f14b63e1a"
    assert message.value() == MESSAGE.deserialized



from pathlib import Path

import pytest

from wunderkafka.tests import TestConsumer, TestHTTPClient
from wunderkafka.serdes.avro import FastAvroDeserializer
from wunderkafka.serdes.headers import ConfluentClouderaHeadersHandler
from wunderkafka.tests.consumer import Message
from wunderkafka.schema_registry import SimpleCache, ConfluentSRClient
from wunderkafka.consumers.constructor import HighLevelDeserializingConsumer
from tests.integration.confluent.conftest import Msg
from wunderkafka.serdes.schemaless.string.deserializers import StringDeserializer

SIGNAL_MESSAGE = Msg(
    payload=b"\x08test\x0cstring\x0cstring\x04NA\xcc\xb8\xeb\xa6\x80_",
    deserialized={
        "source": "test",
        "type": "string",
        "id": "string",
        "value": "NA",
        "ts": 1632128298534,
    },
)


HEADERS = (b"\x00\x00\x00\x08<",)


@pytest.mark.parametrize("header", list(HEADERS))
def test_consume_moving_parts(sr_root_existing: Path, topic: str, header: bytes) -> None:
    msg = Message(topic, value=SIGNAL_MESSAGE.serialized(header), key=b"1632128298534")
    consumer = HighLevelDeserializingConsumer(
        consumer=TestConsumer([msg]),
        schema_registry=ConfluentSRClient(TestHTTPClient(sr_root_existing), SimpleCache()),
        headers_handler=ConfluentClouderaHeadersHandler().parse,
        value_deserializer=FastAvroDeserializer(),
        key_deserializer=StringDeserializer(),
    )

    consumer.subscribe([topic], from_beginning=True)

    messages: list[Message] = consumer.consume()
    [message] = messages
    assert message.key() == "1632128298534"
    assert message.value() == SIGNAL_MESSAGE.deserialized



import uuid
import random
import string
import datetime
from uuid import UUID

import pytest
from pydantic import Field, BaseModel

from wunderkafka.serdes.schemaless.json.serializers import SchemaLessJSONSerializer
from wunderkafka.serdes.schemaless.jsonmodel.serializers import SchemaLessJSONModelSerializer


def get_random_string() -> str:
    return "".join(random.choice(string.printable) for i in range(random.randint(3, 20)))


def get_random_int() -> int:
    return random.randint(-10_000, 10_000)


class RandomName(BaseModel):
    name: str = Field(default_factory=get_random_string)


class MyData(RandomName):
    id: int = Field(default_factory=get_random_int)


class MyUUIDData(RandomName):
    id: UUID = Field(default_factory=uuid.uuid4)


class MyDateTimeData(RandomName):
    created_at: datetime.datetime = Field(default_factory=datetime.datetime.now)


@pytest.fixture(params=[RandomName, MyData, MyUUIDData, MyDateTimeData])
def my_model(request: pytest.FixtureRequest) -> RandomName:
    return request.param()


def test_serialize_schemaless_json(my_model: RandomName) -> None:
    serializer = SchemaLessJSONSerializer()
    serialized = serializer.serialize("", my_model.model_dump())
    assert isinstance(serialized, bytes)


def test_serialize_schemaless_jsonmodel(my_model: RandomName) -> None:
    serializer = SchemaLessJSONModelSerializer()
    serialized = serializer.serialize("", my_model)
    assert isinstance(serialized, bytes)



import json
from typing import Optional
from pathlib import Path

import pytest
from pydantic import UUID4, BaseModel, ValidationError
from pydantic_settings import BaseSettings

from wunderkafka.serdes.avromodel import derive


class NestedModel(BaseModel):
    id: UUID4


def test_simple_base_model() -> None:
    schema = derive(NestedModel, topic="test_data_1")
    assert json.loads(schema) == {
        "type": "record",
        "name": "test_data_1_value",
        "fields": [{"name": "id", "type": {"type": "string", "logicalType": "uuid"}}],
    }


class ResultModel(BaseModel):
    id: UUID4
    nested: list[NestedModel]


def test_nested_base_model_list() -> None:
    schema = derive(ResultModel, topic="test_data_1")
    assert json.loads(schema) == {
        "type": "record",
        "name": "test_data_1_value",
        "fields": [
            {"name": "id", "type": {"type": "string", "logicalType": "uuid"}},
            {
                "name": "nested",
                "type": {
                    "type": "array",
                    "items": {
                        "type": "record",
                        "name": "NestedModel",
                        "fields": [{"name": "id", "type": {"type": "string", "logicalType": "uuid"}}],
                    },
                    "name": "nested",
                },
            },
        ],
    }


class HasDictStrSchema(BaseModel):
    dct: dict[str, NestedModel]


def test_nested_base_model_dict_str_key() -> None:
    schema = derive(HasDictStrSchema, topic="test_data_1")
    assert json.loads(schema) == {
        "type": "record",
        "name": "test_data_1_value",
        "fields": [
            {
                "name": "dct",
                "type": {
                    "type": "map",
                    "values": {
                        "type": "record",
                        "name": "NestedModel",
                        "fields": [{"name": "id", "type": {"type": "string", "logicalType": "uuid"}}],
                    },
                    "name": "dct",
                },
            }
        ],
    }


class HasDictIntSchema(BaseModel):
    dct: dict[int, NestedModel]


def test_nested_base_model_dict_int_key() -> None:
    # InvalidMap derived from Exception =/
    with pytest.raises(Exception):
        derive(HasDictIntSchema, topic="test_data_1")


class Error(BaseModel):
    message: str


class ImageNestedModel(BaseModel):
    id: UUID4
    nested: Optional[NestedModel]
    error: Optional[Error]


def test_nested_base_model_optional() -> None:
    schema = derive(ImageNestedModel, topic="test_data_1")
    assert json.loads(schema) == {
        "type": "record",
        "name": "test_data_1_value",
        "fields": [
            {"name": "id", "type": {"type": "string", "logicalType": "uuid"}},
            {
                "name": "nested",
                "type": [
                    {
                        "type": "record",
                        "name": "NestedModel",
                        "fields": [{"name": "id", "type": {"type": "string", "logicalType": "uuid"}}],
                    },
                    "null",
                ],
            },
            {
                "name": "error",
                "type": [
                    {"type": "record", "name": "Error", "fields": [{"name": "message", "type": "string"}]},
                    "null",
                ],
            },
        ],
    }


class DeepNestedModel(BaseModel):
    result: ResultModel


def test_deep_nested_base_model() -> None:
    schema = derive(DeepNestedModel, topic="test_data_1")
    assert json.loads(schema) == {
        "type": "record",
        "name": "test_data_1_value",
        "fields": [
            {
                "name": "result",
                "type": {
                    "type": "record",
                    "name": "ResultModel",
                    "fields": [
                        {"name": "id", "type": {"type": "string", "logicalType": "uuid"}},
                        {
                            "name": "nested",
                            "type": {
                                "type": "array",
                                "items": {
                                    "type": "record",
                                    "name": "NestedModel",
                                    "fields": [{"name": "id", "type": {"type": "string", "logicalType": "uuid"}}],
                                },
                                "name": "nested",
                            },
                        },
                    ],
                },
            }
        ],
    }


class MLConfig(BaseSettings):
    weights: Path
    threshold: float = 0.5



import pytest

from wunderkafka.errors import DeserializerException
from wunderkafka.serdes.vendors import (
    Vendor,
    Actions,
    Suffixes,
    VendorRegistry,
    ProtocolDefinition,
    get_vendors,
    get_protocol,
    get_protocols,
    get_subject_suffix,
)
from wunderkafka.serdes.structures import Mask, Protocol


@pytest.fixture
def currently_known_protocols_count() -> int:
    return 4


@pytest.fixture
def currently_known_vendors_count() -> int:
    return 2


# Tests for VendorRegistry class still need the fixtures
@pytest.fixture
def sample_vendors() -> list[Vendor]:
    return [
        Vendor(
            name="test_vendor1",
            suffixes=Suffixes("_key1", "_value1"),
            protocols=[
                ProtocolDefinition(id=10, protocol=Protocol(4, Mask("I"))),
                ProtocolDefinition(id=11, protocol=Protocol(8, Mask("q"))),
            ],
        ),
        Vendor(
            name="test_vendor2",
            suffixes=Suffixes("_key2", "_value2"),
            protocols=[
                ProtocolDefinition(id=20, protocol=Protocol(12, Mask("qi"))),
            ],
        ),
    ]


@pytest.fixture
def registry(sample_vendors: list[Vendor]) -> VendorRegistry:
    return VendorRegistry(sample_vendors)


def test_init_and_register_vendors(registry: VendorRegistry) -> None:
    answer = 2
    assert len(registry.get_vendors()) == answer


def test_init_and_register_protocols(registry: VendorRegistry) -> None:
    answer = 3
    assert len(registry.get_protocols()) == answer


@pytest.fixture
def duplicated_vendor(sample_vendors: list[Vendor]) -> list[Vendor]:
    return [
        Vendor(
            name="test_vendor1",
            suffixes=Suffixes("_key3", "_value3"),
            protocols=[ProtocolDefinition(id=30, protocol=Protocol(4, Mask("I")))],
        ),
        *sample_vendors,
    ]


def test_init_with_duplicate_vendor_name(duplicated_vendor: list[Vendor]) -> None:
    with pytest.raises(ValueError, match="Vendor .+ already registered"):
        VendorRegistry(duplicated_vendor)


@pytest.fixture
def duplicated_vendor_protocol(sample_vendors: list[Vendor]) -> list[Vendor]:
    return [
        Vendor(
            name="test_vendor3",
            suffixes=Suffixes("_key3", "_value3"),
            protocols=[ProtocolDefinition(id=10, protocol=Protocol(4, Mask("I")))],
        ),
        *sample_vendors,
    ]


def test_init_with_duplicate_protocol_id(duplicated_vendor_protocol: list[Vendor]) -> None:
    with pytest.raises(ValueError, match="Protocol ID 10 already registered"):
        VendorRegistry(duplicated_vendor_protocol)


def test_get_protocol(registry: VendorRegistry) -> None:
    protocol = registry.get_protocol(10)
    assert isinstance(protocol, Protocol)


def test_get_missing_protocol(registry: VendorRegistry) -> None:
    with pytest.raises(KeyError):
        registry.get_protocol(999)


@pytest.fixture
def suffixes(registry: VendorRegistry) -> Suffixes:
    return registry.get_subject_suffixes(10)


def test_get_subject_suffixes_key(suffixes: Suffixes) -> None:
    assert suffixes.key == "_key1"


def test_get_subject_suffixes_value(suffixes: Suffixes) -> None:
    assert suffixes.value == "_value1"


def test_get_missing_subject_suffixes(registry: VendorRegistry) -> None:
    with pytest.raises(KeyError):
        registry.get_subject_suffixes(999)


@pytest.mark.parametrize("protocol_id", [0, 1, 2, 3])
def test_get_protocol_success(protocol_id: int) -> None:
    protocol = get_protocol(protocol_id, Actions.serialize)
    assert isinstance(protocol, Protocol)


def test_get_protocol_error_deserialize() -> None:
    with pytest.raises(DeserializerException, match="Unknown protocol extracted from message"):
        get_protocol(999, Actions.deserialize)


def test_get_protocol_error_serialize() -> None:
    with pytest.raises(ValueError, match="Unknown protocol_id"):
        get_protocol(999, Actions.serialize)


def test_get_subject_suffix_key_confluent() -> None:
    # Test with confluent protocol (ID=0)
    suffix = get_subject_suffix(0, is_key=True)
    assert suffix == "_key"


def test_get_subject_suffix_value_confluent() -> None:
    suffix = get_subject_suffix(0, is_key=False)
    assert suffix == "_value"


def test_get_subject_suffix_key_cloudera() -> None:
    # Test with cloudera protocol (ID=1)
    suffix = get_subject_suffix(1, is_key=True)
    assert suffix == ":k"


def test_get_subject_suffix_value_cloudera() -> None:
    suffix = get_subject_suffix(1, is_key=False)
    assert suffix == ""


def test_get_subject_suffix_unknown_protocol() -> None:
    with pytest.raises(DeserializerException, match="Unknown protocol extracted"):
        get_subject_suffix(999, is_key=True)


def test_get_protocols_function(currently_known_protocols_count: int) -> None:
    protocols = get_protocols()
    assert len(protocols) == currently_known_protocols_count
    assert all(isinstance(p, Protocol) for p in protocols)


@pytest.fixture
def vendors(currently_known_vendors_count: int) -> dict[str, Vendor]:
    vndrs = get_vendors()
    assert len(vndrs) == currently_known_vendors_count
    return {vendor.name: vendor for vendor in vndrs}


@pytest.mark.parametrize("vendor_name", ["confluent", "cloudera"])
def test_get_vendors_function(vendor_name: str, vendors: dict[str, Vendor]) -> None:
    assert vendor_name in vendors


@pytest.fixture
def confluent(vendors: dict[str, Vendor]) -> Vendor:
    return vendors["confluent"]


def test_confluent_vendor_subject_suffix_key(confluent: Vendor) -> None:
    assert confluent.suffixes.key == "_key"


def test_confluent_vendor_subject_suffix_value(confluent: Vendor) -> None:
    assert confluent.suffixes.value == "_value"


@pytest.fixture
def confluent_protocols(confluent: Vendor) -> dict[int, Protocol]:
    protocols = {pr.id: pr.protocol for pr in confluent.protocols}
    assert len(protocols) == 1
    return protocols


@pytest.mark.parametrize("protocol_id", [0])
def test_confluent_vendor_protocol(protocol_id: int, confluent_protocols: dict[int, Protocol]) -> None:
    assert protocol_id in confluent_protocols


@pytest.fixture
def cloudera(vendors: dict[str, Vendor]) -> Vendor:
    return vendors["cloudera"]


def test_cloudera_vendor_subject_suffix_key(cloudera: Vendor) -> None:
    assert cloudera.suffixes.key == ":k"


def test_cloudera_vendor_subject_suffix_value(cloudera: Vendor) -> None:
    assert cloudera.suffixes.value == ""


@pytest.fixture
def cloudera_protocols(cloudera: Vendor) -> dict[int, Protocol]:
    known_cloudera_protocols = 3
    protocols = {pr.id: pr.protocol for pr in cloudera.protocols}
    assert len(protocols) == known_cloudera_protocols
    return protocols


@pytest.mark.parametrize("protocol_id", [1, 2, 3])
def test_cloudera_vendor_protocol(protocol_id: int, cloudera_protocols: dict[int, Protocol]) -> None:
    assert protocol_id in cloudera_protocols



import sys
import json
from typing import Optional

from pydantic import Field, BaseModel

from wunderkafka.serdes.avromodel import derive

if sys.version_info <= (3, 10):

    class ParentOptional(BaseModel):
        volume: float = Field(description="...")
        weight: Optional[float] = Field(description="...")

    class ChildOptional(ParentOptional):
        ts: int = Field(description="...")

    class GrandsonOptional(ChildOptional): ...

    def test_dataclass() -> None:
        schema = derive(GrandsonOptional, topic="test_data_1")
        assert json.loads(schema) == {
            "type": "record",
            "name": "test_data_1_value",
            "fields": [
                {
                    "name": "volume",
                    "type": "double",
                },
                {
                    "name": "weight",
                    "type": ["double", "null"],
                },
                {
                    "name": "ts",
                    "type": "long",
                },
            ],
        }

else:

    class Parent(BaseModel):
        volume: float = Field(description="...")
        weight: float | None = Field(description="...")

    class Child(Parent):
        ts: int = Field(description="...")

    class Grandson(Child): ...

    def test_dataclass_pipe_annotation() -> None:
        schema = derive(Grandson, topic="test_data_1")
        assert json.loads(schema) == {
            "type": "record",
            "name": "test_data_1_value",
            "fields": [
                {
                    "name": "volume",
                    "type": "double",
                },
                {
                    "name": "weight",
                    "type": ["double", "null"],
                },
                {
                    "name": "ts",
                    "type": "long",
                },
            ],
        }



import datetime

from wunderkafka import BytesConsumer, ConsumerConfig, SecurityProtocol

if __name__ == '__main__':
    # Config is still pretty long, it is an essential complexity.
    # But in minimal setup you only need to specify just `group_id`
    # and `bootstrap_servers`
    config = ConsumerConfig(
        enable_auto_commit=False,
        group_id='my_group',
        bootstrap_servers='kafka-broker-01.my_domain.com:9093',
        security_protocol=SecurityProtocol.sasl_ssl,
        sasl_kerberos_kinit_cmd='kinit my_user@my_realm.com -k -t my_user.keytab',
    )

    consumer = BytesConsumer(config)
    # topic subscription by different timelines is now oneliner without
    # much boilerplate.
    consumer.subscribe(['my_topic'], with_timedelta=datetime.timedelta(hours=10))

    while True:
        msgs = consumer.batch_poll()
        print(
            'Consumed: {0}, errors: {1}'.format(
                len(msgs), len([msg for msg in msgs if msg.error()])
            )
        )



from typing import Optional

from wunderkafka.consumers.bytes import BytesConsumer
from wunderkafka.schema_registry import ClouderaSRClient
from wunderkafka.serdes.headers import ConfluentClouderaHeadersHandler
from wunderkafka.consumers.constructor import HighLevelDeserializingConsumer
from wunderkafka.schema_registry.cache import SimpleCache
from wunderkafka.schema_registry.transport import KerberizableHTTPClient
from wunderkafka.serdes.avro.deserializers import FastAvroDeserializer


def MyAvroConsumer(
    config: Optional[OverridenConfig] = None,
) -> HighLevelDeserializingConsumer:
    config = config or OverridenConfig()
    return HighLevelDeserializingConsumer(
        consumer=BytesConsumer(config),
        schema_registry=ClouderaSRClient(
            KerberizableHTTPClient(config.sr, krb_timeout=10),
            SimpleCache()
        ),
        headers_handler=ConfluentClouderaHeadersHandler().parse,
        deserializer=FastAvroDeserializer(),
    )



from wunderkafka.factories.avro import AvroConsumer, AvroProducer, ConsumerConfig, ProducerConfig, AvroModelProducer
from wunderkafka.config.schema_registry import SRConfig


def test_init_avro_consumer(sr_url: str) -> None:
    AvroConsumer(ConsumerConfig(group_id="test", sr=SRConfig(url=sr_url)))


def test_init_avro_producer(sr_url: str) -> None:
    AvroProducer(config=ProducerConfig(sr=SRConfig(url=sr_url)), mapping={})


def test_init_avro_model_producer(sr_url: str) -> None:
    AvroModelProducer(config=ProducerConfig(sr=SRConfig(url=sr_url)), mapping={})



from __future__ import annotations

import time
from typing import Any, Generic, TypeVar, Optional

from pydantic import Field, BaseModel, ConfigDict, model_validator

M = TypeVar("M")


class PayloadError(BaseModel):
    description: str


class StreamResult(BaseModel, Generic[M]):
    model_config = ConfigDict(arbitrary_types_allowed=True)
    payload: Optional[dict[str, Any]] = None
    error: Optional[PayloadError] = None
    msg: M
    t0: float = Field(default_factory=time.perf_counter)

    @property
    def ok(self) -> bool:
        return self.error is None and self.payload is not None

    @property
    def lifetime(self) -> float:
        return time.perf_counter() - self.t0

    @model_validator(mode="after")
    def verify_mutually_exclusive(self) -> StreamResult:
        # The payload may be None due to Kafka tombstones:
        # https://docs.confluent.io/kafka/design/log_compaction.html
        # In short, these are messages with a null value, used to indicate that their key has been deleted.
        if self.payload is not None and self.error is not None:
            raise ValueError("payload and error cannot both be not None")
        return self



from wunderkafka import ConsumerConfig, SRConfig, SecurityProtocol, AvroConfluentConsumer

BROKERS_ADDRESSES = 'kafka-broker-01.my_domain.com'
SCHEMA_REGISTRY_URL = 'https://schema-registry.my_domain.com'

if __name__ == '__main__':
    config = ConsumerConfig(
        enable_auto_commit=False,
        group_id='my_group',
        bootstrap_servers=BROKERS_ADDRESSES,
        security_protocol=SecurityProtocol.sasl_ssl,
        sasl_kerberos_kinit_cmd='kinit my_user@my_real.com -k -t my_user.keytab',
        sr=SRConfig(url=SCHEMA_REGISTRY_URL, sasl_username='my_user@my_real.com'),
    )

    c = AvroConfluentConsumer(
        config=config,
    )
    c.subscribe(['my_topic'], from_beginning=True)

    while True:
        msgs = c.consume(timeout=10.0, ignore_keys=True)
        print(len(msgs))



import uuid
from uuid import UUID
from typing import Optional

import pytest
from pydantic import BaseModel

from wunderkafka.serdes.json import HAS_JSON_SCHEMA
from wunderkafka.serdes.schemaless.string.serializers import StringSerializer
from wunderkafka.serdes.schemaless.jsonmodel.serializers import SchemaLessJSONModelSerializer

if not HAS_JSON_SCHEMA:
    pytest.skip("skipping json-schema-only tests", allow_module_level=True)


from wunderkafka.tests import TestProducer
from wunderkafka.producers.constructor import HighLevelSerializingProducer


class Image(BaseModel):
    id: Optional[UUID] = None
    path: Optional[str] = None


def test_json_producer_string_key_create_schema() -> None:
    topic = "testing_json_str_producer"
    test_producer = TestProducer()
    producer = HighLevelSerializingProducer(
        producer=test_producer,
        schema_registry=None,
        header_packer=None,
        value_serializer=SchemaLessJSONModelSerializer(),
        key_serializer=StringSerializer(),
        mapping={topic: (Image, str)},
        protocol_id=0,
    )

    key = "714fc713-37ff-4477-9157-cb4f14b63e1a"
    value = Image(
        id=uuid.UUID(key),
        path="/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3",
    )

    producer.send_message(topic, value, key)

    [message] = test_producer.sent

    assert message.key == b"714fc713-37ff-4477-9157-cb4f14b63e1a"
    assert (
        message.value
        == b'{"id":"714fc713-37ff-4477-9157-cb4f14b63e1a","path":"/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3"}'
    )  # noqa: E501



from wunderkafka import AvroProducer, ProducerConfig, SRConfig, SecurityProtocol

BROKERS_ADDRESSES = 'kafka-broker-01.my_domain.com'
SCHEMA_REGISTRY_URL = 'https://schema-registry.my_domain.com'

value_schema = """
{
   "namespace": "my.test",
   "name": "value",
   "type": "record",
   "fields" : [
     {
       "name" : "name",
       "type" : "string"
     }
   ]
}
"""
key_schema = """
{
   "namespace": "my.test",
   "name": "key",
   "type": "record",
   "fields" : [
     {
       "name" : "name",
       "type" : "string"
     }
   ]
}
"""

if __name__ == '__main__':
    config = ProducerConfig(
        bootstrap_servers=BROKERS_ADDRESSES,
        security_protocol=SecurityProtocol.sasl_ssl,
        sasl_kerberos_kinit_cmd='kinit my_user@my_real.com -k -t my_user.keytab',
        sr=SRConfig(url=SCHEMA_REGISTRY_URL, sasl_username='my_user@my_real.com'),
    )

    topic = 'test_test_test'
    producer = AvroProducer({topic: (value_schema, key_schema)}, config)
    producer.send_message(topic, {"name": "Value"}, {"name": "Key"}, blocking=True)



from typing import Any
from pathlib import Path
from dataclasses import dataclass

import pytest


@pytest.fixture
def topic() -> str:
    return "testing_wunderkafka"


@pytest.fixture
def sr_root(fixtures_root: Path) -> Path:
    return fixtures_root / "schema_registry" / "confluent"


@pytest.fixture
def sr_root_create(sr_root: Path) -> Path:
    return sr_root / "create"


@pytest.fixture
def sr_root_existing(sr_root: Path) -> Path:
    return sr_root / "existing"


@pytest.fixture
def sr_root_update(sr_root: Path) -> Path:
    return sr_root / "update"


@dataclass
class Msg:
    payload: bytes
    deserialized: dict[str, Any]

    def serialized(self, header: bytes) -> bytes:
        return header + self.payload



from pydantic import BaseModel, Field
from wunderkafka import AvroModelProducer, ProducerConfig, SRConfig, SecurityProtocol
from wunderkafka.time import now

BROKERS_ADDRESSES = 'kafka-broker-01.my_domain.com'
SCHEMA_REGISTRY_URL = 'https://schema-registry.my_domain.com'


class SomeEvent(BaseModel):
    name: str = 'test'
    ts: int = Field(default_factory=now)

    class Meta:
        name = 'my.test'


if __name__ == '__main__':
    config = ProducerConfig(
        bootstrap_servers=BROKERS_ADDRESSES,
        security_protocol=SecurityProtocol.sasl_ssl,
        sasl_kerberos_kinit_cmd='kinit my_user@my_real.com -k -t my_user.keytab',
        sr=SRConfig(url=SCHEMA_REGISTRY_URL, sasl_username='my_user@my_real.com'),
    )

    topic = 'test_test_test'

    # No key, that's just an example
    producer = AvroModelProducer({topic: SomeEvent}, config)
    producer.send_message(topic, SomeEvent(), blocking=True)



from uuid import UUID
from typing import Optional
from pathlib import Path

from pydantic import UUID4, BaseModel

from wunderkafka.tests import TestProducer, TestHTTPClient
from wunderkafka.serdes.store import AvroModelRepo
from wunderkafka.serdes.headers import ConfluentClouderaHeadersHandler
from wunderkafka.schema_registry import SimpleCache, ConfluentSRClient
from wunderkafka.producers.constructor import HighLevelSerializingProducer
from wunderkafka.serdes.avromodel.serializers import AvroModelSerializer
from wunderkafka.serdes.schemaless.string.serializers import StringSerializer


class Image(BaseModel):
    id: UUID4
    path: Optional[str] = None


def test_avro_producer_string_key_create_schema(sr_root_existing: Path) -> None:
    topic = "testing_avro_str_producer"
    test_producer = TestProducer()
    sr_client = ConfluentSRClient(TestHTTPClient(sr_root_existing), SimpleCache())
    producer = HighLevelSerializingProducer(
        producer=test_producer,
        schema_registry=sr_client,
        header_packer=ConfluentClouderaHeadersHandler().pack,
        value_serializer=AvroModelSerializer(AvroModelRepo()),
        key_serializer=StringSerializer(),
        mapping={topic: (Image, str)},
        protocol_id=0,
    )

    key = "714fc713-37ff-4477-9157-cb4f14b63e1a"
    value = Image(
        id=UUID("714fc713-37ff-4477-9157-cb4f14b63e1a"),
        path="/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3",
    )

    producer.send_message(topic, value, key)

    [message] = test_producer.sent

    assert message.key == b"714fc713-37ff-4477-9157-cb4f14b63e1a"
    assert (
        message.value
        == b"\x00\x00\x00\x00\x13H714fc713-37ff-4477-9157-cb4f14b63e1a\x02x/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3"
    )  # noqa: E501



import datetime

from wunderkafka import AvroConsumer, ConsumerConfig, SRConfig, SecurityProtocol, TopicSubscription

BROKERS_ADDRESSES = 'kafka-broker-01.my_domain.com'
SCHEMA_REGISTRY_URL = 'https://schema-registry.my_domain.com'


if __name__ == '__main__':
    config = ConsumerConfig(
        enable_auto_commit=False,
        group_id='my_group',
        bootstrap_servers=BROKERS_ADDRESSES,
        security_protocol=SecurityProtocol.sasl_ssl,
        sasl_kerberos_kinit_cmd='kinit my_user@my_real.com -k -t my_user.keytab',
        sr=SRConfig(url=SCHEMA_REGISTRY_URL, sasl_username='my_user@my_real.com'),
    )

    c = AvroConsumer(config)
    c.subscribe([
        TopicSubscription('my_topic1', with_timedelta=datetime.timedelta(minutes=10)),
        TopicSubscription('my_topic2', from_beginning=True),
        # Will also work with "default" subscription option
        'my_topic3',
    ])

    while True:
        msgs = c.consume(timeout=10.0, ignore_keys=True)
        print(len(msgs))



import datetime

from my_module import MyConsumer


if __name__ == '__main__':
    consumer = MyConsumer()
    consumer.subscribe(['my_topic'], with_timedelta=datetime.timedelta(minutes=2))
    while True:
        msgs = consumer.consume()
        ...



import json

from pydantic import UUID4
from dataclasses_avroschema.pydantic import AvroBaseModel

from wunderkafka.serdes.avromodel import derive


class AvroNestedClass(AvroBaseModel):
    id: UUID4


class AvroResultClass(AvroBaseModel):
    id: UUID4
    nested: list[AvroNestedClass]


class AvroSimilarImage(AvroBaseModel):
    id: UUID4


def test_avro_model() -> None:
    schema = derive(AvroSimilarImage, topic="test_data_1")
    assert json.loads(schema) == {
        "type": "record",
        "name": "test_data_1_value",
        "fields": [{"name": "id", "type": {"type": "string", "logicalType": "uuid"}}],
    }


def test_avro_base_model() -> None:
    schema = derive(AvroResultClass, topic="test_data_1")
    assert json.loads(schema) == {
        "type": "record",
        "name": "test_data_1_value",
        "fields": [
            {"name": "id", "type": {"type": "string", "logicalType": "uuid"}},
            {
                "name": "nested",
                "type": {
                    "type": "array",
                    "items": {
                        "type": "record",
                        "name": "AvroNestedClass",
                        "fields": [{"name": "id", "type": {"type": "string", "logicalType": "uuid"}}],
                    },
                    "name": "nested",
                },
            },
        ],
    }



import json

from pydantic import BaseModel

from wunderkafka.serdes.avromodel import derive


class MyModel(BaseModel):
    a: int
    b: str


def test_derive_hyphens(topic: str = "my-topic") -> None:
    schema = derive(MyModel, topic=topic)
    assert json.loads(schema) == {
        "name": "my_topic_value",
        "type": "record",
        "fields": [
            {"name": "a", "type": "long"},
            {"name": "b", "type": "string"},
        ],
    }


def test_derive_leading_underscores(topic: str = "_my-topic") -> None:
    schema = derive(MyModel, topic=topic)
    assert json.loads(schema) == {
        "name": "my_topic_value",
        "type": "record",
        "fields": [
            {"name": "a", "type": "long"},
            {"name": "b", "type": "string"},
        ],
    }



from pathlib import Path

import pytest

from wunderkafka.config.generate import (
    Row,
    group,
    parse,
    read_markdown,
    generate_enums,
    generate_fields,
    generate_models,
)


def read_answer(root: Path, file_name: str) -> list[str]:
    with open(root / "answers" / file_name) as fl:
        return fl.read().split("\n")[:-1]


@pytest.fixture
def fixture_root() -> Path:
    return Path(__file__).parent.parent / "fixtures" / "config"


@pytest.fixture
def configuration_md(fixture_root: Path) -> Path:
    return fixture_root / "1.5.0" / "CONFIGURATION.md"


@pytest.fixture
def enums(fixture_root: Path) -> list[str]:
    return read_answer(fixture_root, "enums.py")


@pytest.fixture
def fields(fixture_root: Path) -> list[str]:
    return read_answer(fixture_root, "fields.py")


@pytest.fixture
def models(fixture_root: Path) -> list[str]:
    return read_answer(fixture_root, "models.py")


@pytest.fixture
def grouped(configuration_md: Path) -> dict[str, list[Row]]:
    return group(parse(read_markdown(filename=configuration_md)))


def test_enums(grouped: dict[str, list[Row]], enums: list[str]) -> None:
    generated = generate_enums(grouped)
    assert enums == generated


def test_fields(grouped: dict[str, list[Row]], fields: list[str]) -> None:
    generated = generate_fields(grouped)
    assert fields == generated


def test_models(grouped: dict[str, list[Row]], models: list[str]) -> None:
    generated = generate_models(grouped)
    # Multiline for builtin
    assert models == "\n".join(generated).split("\n")



import os
from functools import partial

from pydantic import field_validator, Field
from wunderkafka.time import now
from wunderkafka import SRConfig, ConsumerConfig, SecurityProtocol, AvroConsumer


# If you are a fan of 12 factors, you may want to config via env variables
class OverridenSRConfig(SRConfig):
    url: str = Field(alias='SCHEMA_REGISTRY_URL')

    @field_validator('sasl_username')
    @classmethod
    def from_env(cls, v) -> str:
        # And to use 'native' kerberos envs
        return '{0}@{1}'.format(os.environ.get('KRB5_USER'), os.environ.get('KRB5_REALM'))


# Or you want to override some defaults by default (pun intended)
class OverridenConfig(ConsumerConfig):
    # Consumer which do not commit messages automatically
    enable_auto_commit: bool = False
    # And knows nothing after restart due to new gid.
    group_id: str = 'wunderkafka-{0}'.format(now())
    # More 12 factors
    bootstrap_servers: str = Field(env='BOOTSTRAP_SERVER')
    security_protocol: SecurityProtocol = SecurityProtocol.sasl_ssl
    sasl_kerberos_kinit_cmd: str = ''
    sr: SRConfig = OverridenSRConfig()

    @field_validator('sasl_kerberos_kinit_cmd')
    @classmethod
    def format_keytab(cls, v) -> str:
        if not v:
            return 'kinit {0}@{1} -k -t {0}.keytab'.format(os.environ.get('KRB5_USER'), os.environ.get('KRB5_REALM'))
        # Still allowing to set it manually
        return str(v)


# After this, you can `partial` your own Producer/Consumer, something like...
MyConsumer = partial(AvroConsumer, config=OverridenConfig())



from typing import Optional
from pathlib import Path

from pydantic import BaseModel

from wunderkafka.tests import TestProducer, TestHTTPClient
from wunderkafka.serdes.avro import AvroModelSerializer
from wunderkafka.serdes.store import AvroModelRepo
from wunderkafka.serdes.headers import ConfluentClouderaHeadersHandler
from wunderkafka.schema_registry import SimpleCache, ConfluentSRClient
from wunderkafka.producers.constructor import HighLevelSerializingProducer


class Event(BaseModel):
    id: Optional[int]
    ts: Optional[int]

    class Meta:
        namespace = "any.data"


class EvolvedEvent(Event):
    description: Optional[str] = None
    info: Optional[str] = "test"


def test_avro_producer_create_schema(sr_root_create: Path, topic: str, schema_description: type[Event] = Event) -> None:
    fixed_ts = 1632128298534
    test_producer = TestProducer()
    producer = HighLevelSerializingProducer(
        producer=test_producer,
        schema_registry=ConfluentSRClient(TestHTTPClient(sr_root_create), SimpleCache()),
        header_packer=ConfluentClouderaHeadersHandler().pack,
        serializer=AvroModelSerializer(),
        store=AvroModelRepo(),
        mapping={topic: schema_description},
        protocol_id=0,
    )

    key = None
    value = schema_description(id=None, ts=fixed_ts)

    producer.send_message(topic, value, key)

    [message] = test_producer.sent

    assert message.key is None
    assert message.value == b"\x00\x00\x00\x00\x14\x02\x00\xcc\xb8\xeb\xa6\x80_"


def test_avro_producer_existing_schema(
    sr_root_existing: Path,
    topic: str,
    schema_description: type[Event] = Event,
) -> None:
    fixed_ts = 1632128298534
    test_producer = TestProducer()
    producer = HighLevelSerializingProducer(
        producer=test_producer,
        schema_registry=ConfluentSRClient(TestHTTPClient(sr_root_existing), SimpleCache()),
        header_packer=ConfluentClouderaHeadersHandler().pack,
        serializer=AvroModelSerializer(),
        store=AvroModelRepo(),
        mapping={topic: schema_description},
        protocol_id=0,
    )

    key = None
    value = schema_description(id=None, ts=fixed_ts)

    producer.send_message(topic, value, key)

    [message] = test_producer.sent

    assert message.key is None
    assert message.value == b"\x00\x00\x00\x00\x14\x02\x00\xcc\xb8\xeb\xa6\x80_"


def test_avro_producer_update_schema(
    sr_root_update: Path,
    topic: str,
    schema_description: type[EvolvedEvent] = EvolvedEvent,
) -> None:
    fixed_ts = 1632128298534
    test_producer = TestProducer()
    producer = HighLevelSerializingProducer(
        producer=test_producer,
        schema_registry=ConfluentSRClient(TestHTTPClient(sr_root_update), SimpleCache()),
        header_packer=ConfluentClouderaHeadersHandler().pack,
        serializer=AvroModelSerializer(),
        store=AvroModelRepo(),
        mapping={topic: schema_description},
        protocol_id=0,
    )

    key = None
    value = schema_description(id=None, ts=fixed_ts)

    producer.send_message(topic, value, key)

    [message] = test_producer.sent

    assert message.key is None
    assert message.value == b"\x00\x00\x00\x00\x15\x02\x00\xcc\xb8\xeb\xa6\x80_\x00\x00\x08test"



from uuid import UUID
from typing import Optional
from pathlib import Path

import pytest
from pydantic import UUID4, BaseModel

from wunderkafka.serdes.json import HAS_JSON_SCHEMA
from wunderkafka.serdes.store import JSONModelRepo

if not HAS_JSON_SCHEMA:
    pytest.skip("skipping json-schema-only tests", allow_module_level=True)
from wunderkafka.tests import TestProducer, TestHTTPClient
from wunderkafka.serdes.headers import ConfluentClouderaHeadersHandler
from wunderkafka.schema_registry import SimpleCache, ConfluentSRClient
from wunderkafka.producers.constructor import HighLevelSerializingProducer
from wunderkafka.serdes.jsonmodel.serializers import JSONModelSerializer


class Image(BaseModel):
    id: Optional[UUID4] = None
    path: Optional[str] = None


def test_json_producer_create_schema(sr_root_existing: Path) -> None:
    topic = "testing_json_str_producer"
    test_producer = TestProducer()
    sr_client = ConfluentSRClient(TestHTTPClient(sr_root_existing), SimpleCache())
    producer = HighLevelSerializingProducer(
        producer=test_producer,
        schema_registry=sr_client,
        header_packer=ConfluentClouderaHeadersHandler().pack,
        serializer=JSONModelSerializer(sr_client.client),
        store=JSONModelRepo(),
        mapping={topic: Image},
        protocol_id=0,
    )

    key = None
    value = Image(
        id=UUID("714fc713-37ff-4477-9157-cb4f14b63e1a"),
        path="/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3",
    )

    producer.send_message(topic, value, key)

    [message] = test_producer.sent

    assert message.key is None
    assert (
        message.value
        == b'\x00\x00\x00\x07<{"id": "714fc713-37ff-4477-9157-cb4f14b63e1a", "path": "/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3"}'
    )  # noqa: E501



import json
from typing import Any
from pathlib import Path

import pytest

from wunderkafka.structures import SchemaType
from wunderkafka.serdes.store import SchemaFSRepo


@pytest.fixture
def repo() -> SchemaFSRepo:
    return SchemaFSRepo(schema_type=SchemaType.AVRO)


@pytest.fixture
def fs_root_dir(fixtures_root: Path) -> Path:
    return fixtures_root / "fs_repo"


@pytest.fixture
def answer_json() -> dict[str, Any]:
    return {
        "type": "record",
        "name": "exampleSchema",
        "fields": [
            {"name": "name", "type": "string", "default": ""},
            {"name": "value", "type": "string", "default": ""},
        ],
    }


@pytest.mark.parametrize("key_schema_path,value_schema_path", [("key.avsc", "value.avsc")])
def test_fs_repo(
    repo: SchemaFSRepo,
    topic: str,
    key_schema_path: str,
    value_schema_path: str,
    fs_root_dir: Path,
    answer_json: dict[str, Any],
) -> None:
    repo.add(topic, fs_root_dir / value_schema_path, fs_root_dir / key_schema_path)
    key_schema = repo.get(topic, is_key=True)
    assert key_schema is not None
    assert key_schema.text == '"long"'
    value_schema = repo.get(topic)
    assert value_schema is not None
    assert json.loads(value_schema.text) == answer_json



from typing import Any

import pytest
from pytest import FixtureRequest

from wunderkafka import SecurityProtocol

RawConfig = dict[str, Any]


@pytest.fixture
def boostrap_servers() -> str:
    return "localhost:9093"


@pytest.fixture
def default_config(boostrap_servers: str) -> RawConfig:
    return {"bootstrap_servers": boostrap_servers}


@pytest.fixture
def non_krb_config(boostrap_servers: str) -> RawConfig:
    return {
        "bootstrap_servers": boostrap_servers,
        "sasl_username": "user",
        "sasl_password": "password",
        "security_protocol": SecurityProtocol.sasl_ssl,
        "sasl_mechanism": "SCRAM-SHA-512",
    }


@pytest.fixture(params=[SecurityProtocol.sasl_ssl, SecurityProtocol.sasl_plaintext])
def krb_config(request: FixtureRequest, boostrap_servers: str) -> RawConfig:
    return {
        "bootstrap_servers": boostrap_servers,
        "sasl_kerberos_keytab": "user.keytab",
        "security_protocol": request.param,
    }
